function V = ComputeStepDirection
% Uses IterInfo.r, IterInfo.W and IterInfo.pi

global piDefs isRezDefs IterInfo

% V = zeros(piDefs.pairCount,1);
% for ij=piDefs.Pr{IterInfo.r}
%     V(ij) = 1;
% end
% return

r = IterInfo.r;
V = zeros([piDefs.pairCount 1]);

% Computing \Psi^{(1,1)}
Psi11 = zeros(piDefs.pairCount, piDefs.pairCount);
for mn=1:piDefs.pairCount
    % Compute Pi-bars-1 and Pi-1,0
    PiBar1 = getPiBar1(mn);
    Pi10 = getPi10(mn);

    for ij=piDefs.Pr{r}
        
        k = setdiff(IP(ij),r);
        Psi11(ij, mn) = Omg(IterInfo.pi{k}, IterInfo.pi{r}, isRezDefs{ij}, 1, 0)' * (PiBar1{k}-PiBar1{r}+Pi10);
        
    end
end

%Psi11,
%pause;

[vv, dd] = eig(Psi11);
dvec = (dd*ones(piDefs.pairCount,1))';

tol = 1e-8;
d_isEigvalPositive = (dvec>0);
zeroPoses = setdiff((1:piDefs.pairCount), piDefs.Pr{r})';
for a=1:piDefs.pairCount
    
    if (all(abs(vv(zeroPoses,a))<=tol))
        d_isOtherIncZero(a) = true;
    else
        d_isOtherIncZero(a) = false;
    end
    
    if (all(vv(:,a)>=-tol))
        d_isAnyIncNegative(a) = false;
    elseif (all(-vv(:,a)>=-tol))
        d_isAnyIncNegative(a) = false;
        vv(:,a) = -vv(:,a);
    else
        d_isAnyIncNegative(a) = true;
    end
    
end

d_isValid = (d_isEigvalPositive & d_isOtherIncZero & ~d_isAnyIncNegative);

V = vv(:,find(d_isValid,1));

if isempty(V)
    disp(sprintf('WARNING: V is empty!'));
    r,
    vv,
    dvec,
    d_isEigvalPositive,
    d_isOtherIncZero,
    d_isAnyIncNegative,
    d_isValid,
    colNo = input('Enter the index of the column to use: ');
    V = vv(:, colNo);
end

% ---------------------------------

% M and N can be precomputed

function Mmat = M(r) % uses IterInfo.W and IterInfo.pi
global IterInfo isRezDefs piDefs
Mmat = c([piDefs.starts(r); IterInfo.pi{r}; piDefs.ends(r)], 2);
for lr=piDefs.Pr{r}
    l = setdiff(IP(lr), r);
    Mmat = Mmat + IterInfo.W(lr)*Omg(IterInfo.pi{l}, IterInfo.pi{r}, isRezDefs{lr}, 0, 2);
end

% ---------------------------------

function Nmat = N(lr) % uses isRezDefs and IterInfo.pi
global IterInfo isRezDefs
[l, r] = IP(lr);
Nmat = Omg(IterInfo.pi{l}, IterInfo.pi{r}, isRezDefs{lr}, 1, 1);

% ---------------------------------

function PiBar1 = getPiBar1(mn)
global piDefs isRezDefs IterInfo
[m, n] = IP(mn);

BigMat = [];
BigVec = [];
for r=1:piDefs.N
    
    BigRow = [];
    for k=1:piDefs.N
        if k==r
            BigRow = [BigRow, M(r)];
        else
            kr = IP(k,r);
            BigRow = [BigRow, IterInfo.W(kr)*N(kr)];
        end
    end
    BigMat = [BigMat; BigRow];
    
    if r==m
        BigVec = [BigVec; Omg(IterInfo.pi{m}, IterInfo.pi{n}, isRezDefs{mn}, 1, 0)];
    elseif r==n
        BigVec = [BigVec; Omg(IterInfo.pi{n}, IterInfo.pi{m}, isRezDefs{mn}, 1, 0)];
    else
        BigVec = [BigVec; zeros(piDefs.L, 1)];
    end
end

PiBar1Vec = inv(BigMat) * BigVec;
for r=1:piDefs.N
    PiBar1{r} = PiBar1Vec(((r-1)*piDefs.L+1):(r*piDefs.L));
end

% ---------------------------------

function Pi10 = getPi10(mn) % uses IterInfo.r
global IterInfo isRezDefs piDefs
r = IterInfo.r;
[m, n] = IP(mn);
if m==r || n==r
    k = setdiff([m n], r);
    Pi10 = inv(M(r)) * Omg(IterInfo.pi{k}, IterInfo.pi{r}, isRezDefs{mn}, 1, 0);
else
    Pi10 = zeros(piDefs.L, 1);
end
