function res = Omg(pi1, pi2, isRez, diffOrd1, diffOrd2)
% pi is a column vector
% In the folllowing computation pi is assumed to have the form
% [y1 y2 ... yN]'
%
% rezInds is a vector of 1's & 0's of the form
% [i1 i2 ... iN]'
%
if nargin < 4
    diffOrd1 = 0;
    diffOrd2 = 0;
end

% ------------------------------
clear global comp_omg
global comp_omg

comp_omg.count = length(pi1);
comp_omg.isActive = (isRez==1);
comp_omg.count_active = sum(comp_omg.isActive);
comp_omg.dy_active = zeros(size(comp_omg.isActive));
comp_omg.dy_active(comp_omg.isActive) = pi1(comp_omg.isActive) - pi2(comp_omg.isActive);
comp_omg.dySq_active = zeros(size(comp_omg.isActive));
comp_omg.dySq_active(comp_omg.isActive) = comp_omg.dy_active(comp_omg.isActive).^2;
comp_omg.dySqSum = sum(comp_omg.dySq_active(comp_omg.isActive));
comp_omg.dySqSumSqrt = sqrt(comp_omg.dySqSum);

% ------------------------------

if diffOrd1==0 && diffOrd2==0
    res = omg00;
elseif diffOrd1==1 && diffOrd2==0
    res = omg10;
elseif diffOrd1==0 && diffOrd2==1
    res = omg01;
elseif diffOrd1==1 && diffOrd2==1
    res = omg11;
elseif diffOrd1==0 && diffOrd2==2
    res = omg02;
elseif diffOrd1==2 && diffOrd2==0
    res = omg20;
end

% ========================================================

function res = omg00
global comp_omg
res = comp_omg.dySqSumSqrt;

% ========================================================

function res = omg10
global comp_omg

for j = 1:comp_omg.count
    if (~comp_omg.isActive(j))
        res(j) = 0;
    else
        res(j) = comp_omg.dy_active(j) / comp_omg.dySqSumSqrt;
        if isnan(res(j))
            res(j) = 0;
        end
    end
end

res = res';

% ------------------------------

function res = omg01
global comp_omg

for j = 1:comp_omg.count
    if (~comp_omg.isActive(j))
        res(j) = 0;
    else
        res(j) = -comp_omg.dy_active(j) / comp_omg.dySqSumSqrt;
        if isnan(res(j))
            res(j) = 0;
        end
    end
end

res = res';

% ------------------------------

function res = omg11
global comp_omg

for j = 1:comp_omg.count
    for k = j:comp_omg.count
        if (~comp_omg.isActive(j) || ~comp_omg.isActive(j))
            res(j,k) = 0;
        elseif j==k
            res(j,k) = -(comp_omg.dySqSum - comp_omg.dySq_active(j)) / (comp_omg.dySqSumSqrt*comp_omg.dySqSum);
            if isnan(res(j,k))
                res(j,k) = 0;
            end
        else
            res(j,k) = comp_omg.dy_active(j) * comp_omg.dy_active(k) / (comp_omg.dySqSumSqrt*comp_omg.dySqSum);
            if isnan(res(j,k))
                res(j,k) = 0;
            end
        end
        res(k,j) = res(j,k);
    end
end

% ------------------------------

function res = omg20
global comp_omg

for j = 1:comp_omg.count
    for k = j:comp_omg.count
        if (~comp_omg.isActive(j) || ~comp_omg.isActive(j))
            res(j,k) = 0;
        elseif j==k
            res(j,k) = (comp_omg.dySqSum - comp_omg.dySq_active(j)) / (comp_omg.dySqSumSqrt*comp_omg.dySqSum);
            if isnan(res(j,k))
                res(j,k) = 0;
            end
        else
            res(j,k) = -comp_omg.dy_active(j) * comp_omg.dy_active(k) / (comp_omg.dySqSumSqrt*comp_omg.dySqSum);
            if isnan(res(j,k))
                res(j,k) = 0;
            end
        end
        res(k,j) = res(j,k);
    end
end

% ------------------------------

function res = omg02
global comp_omg

for j = 1:comp_omg.count
    for k = j:comp_omg.count
        if (~comp_omg.isActive(j) || ~comp_omg.isActive(j))
            res(j,k) = 0;
        elseif j==k
            res(j,k) = (comp_omg.dySqSum - comp_omg.dySq_active(j)) / (comp_omg.dySqSumSqrt*comp_omg.dySqSum);
            if isnan(res(j,k))
                res(j,k) = 0;
            end
        else
            res(j,k) = -comp_omg.dy_active(j) * comp_omg.dy_active(k) / (comp_omg.dySqSumSqrt*comp_omg.dySqSum);
            if isnan(res(j,k))
                res(j,k) = 0;
            end
        end
        res(k,j) = res(j,k);
    end
end

% ========================================================
